package com.jiuqi.script.rhino;

import com.jiuqi.script.IScriptObject;
import org.mozilla.javascript.Callable;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

public class RhinoScriptObject extends ScriptableObject {
   public static final String CLASSNAME = "CustomScriptObject";
   private IScriptObject sobj;

   public RhinoScriptObject(IScriptObject sobj) {
      this.sobj = sobj;
   }

   public String getClassName() {
      return "CustomScriptObject";
   }

   public IScriptObject getScriptObject() {
      return this.sobj;
   }

   public Object[] getAllIds() {
      return this.getIds();
   }

   public Object[] getIds() {
      String[] properties = this.sobj.getPropertyNames();
      String[] functions = this.sobj.getFunctionNames();
      String[] ids = new String[properties.length + functions.length];
      System.arraycopy(properties, 0, ids, 0, properties.length);
      System.arraycopy(functions, 0, ids, properties.length, functions.length);
      return ids;
   }

   public Object get(String name, Scriptable start) {
      return this.sobj.hasFunction(name) ? new ScriptFunction(name, this.sobj) : this.sobj.getProperty(name);
   }

   public Object get(int index, Scriptable start) {
      String[] properties = this.sobj.getPropertyNames();
      return properties.length > index ? this.sobj.getProperty(this.sobj.getPropertyNames()[index]) : null;
   }

   public boolean has(String name, Scriptable start) {
      return this.sobj.hasProperty(name);
   }

   public boolean has(int index, Scriptable start) {
      String[] properties = this.sobj.getPropertyNames();
      return properties.length > index ? this.sobj.hasProperty(this.sobj.getPropertyNames()[index]) : false;
   }

   public void put(String name, Scriptable start, Object value) {
      this.sobj.setProperty(name, value);
   }

   public void delete(String name) {
      this.sobj.delProperty(name);
   }

   public void delete(int index) {
      String[] properties = this.sobj.getPropertyNames();
      if (properties.length > index) {
         this.sobj.delProperty(this.sobj.getPropertyNames()[index]);
      }

   }

   private class ScriptFunction implements Callable {
      private String functionName;
      private IScriptObject scriptObject;

      public ScriptFunction(String functionName, IScriptObject scriptObject) {
         this.functionName = functionName;
         this.scriptObject = scriptObject;
      }

      public Object call(Context cx, Scriptable scope, Scriptable thisObj, Object[] args) {
         if (args == null) {
            return this.scriptObject.invoke(this.functionName, new Object[0]);
         } else {
            Object[] newArgs = new Object[args.length];

            for(int i = 0; i < args.length; ++i) {
               newArgs[i] = ScriptHelper._toJava(args[i], scope);
            }

            return this.scriptObject.invoke(this.functionName, newArgs);
         }
      }
   }
}
